use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'bin/abc-path-solve',
    'lib/Games/ABC_Path/Solver.pm',
    'lib/Games/ABC_Path/Solver/App.pm',
    'lib/Games/ABC_Path/Solver/Base.pm',
    'lib/Games/ABC_Path/Solver/Board.pm',
    'lib/Games/ABC_Path/Solver/Constants.pm',
    'lib/Games/ABC_Path/Solver/Coord.pm',
    'lib/Games/ABC_Path/Solver/Move.pm',
    'lib/Games/ABC_Path/Solver/Move/LastRemainingCellForLetter.pm',
    'lib/Games/ABC_Path/Solver/Move/LastRemainingLetterForCell.pm',
    'lib/Games/ABC_Path/Solver/Move/LettersNotInVicinity.pm',
    'lib/Games/ABC_Path/Solver/Move/ResultsInASuccess.pm',
    'lib/Games/ABC_Path/Solver/Move/ResultsInAnError.pm',
    'lib/Games/ABC_Path/Solver/Move/TryingLetterForCell.pm',
    't/00-compile.t',
    't/00-load.t',
    't/input-from-string.t',
    't/layouts/brain-bashers.2010-12-21.abc-path',
    't/layouts/brain-bashers.2010-12-22.abc-path',
    't/results/brain-bashers.2010-12-21.abc-path-sol',
    't/results/brain-bashers.2010-12-22.abc-path-sol',
    't/solver-output-system-test.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
